# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ai/h2o/featurestore/api/v1/FeatureSetProtoApi.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n3ai/h2o/featurestore/api/v1/FeatureSetProtoApi.proto\x12\x1a\x61i.h2o.featurestore.api.v1\"[\n\x10\x46\x65\x61tureSetHeader\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x02 \x01(\t\x12\x1b\n\x13\x66\x65\x61ture_set_version\x18\x03 \x01(\t*)\n\x0e\x46\x65\x61tureSetType\x12\x07\n\x03RAW\x10\x00\x12\x0e\n\nENGINEERED\x10\x01*m\n\x0b\x46\x65\x61tureType\x12\x17\n\x13\x41UTOMATIC_DISCOVERY\x10\x00\x12\r\n\tCOMPOSITE\x10\x01\x12\r\n\tNUMERICAL\x10\x02\x12\x0f\n\x0b\x43\x41TEGORICAL\x10\x03\x12\x0c\n\x08TEMPORAL\x10\x04\x12\x08\n\x04TEXT\x10\x05\x42\x19\x42\x17\x46\x65\x61tureSetProtoApiProtob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ai.h2o.featurestore.api.v1.FeatureSetProtoApi_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\027FeatureSetProtoApiProto'
  _globals['_FEATURESETTYPE']._serialized_start=176
  _globals['_FEATURESETTYPE']._serialized_end=217
  _globals['_FEATURETYPE']._serialized_start=219
  _globals['_FEATURETYPE']._serialized_end=328
  _globals['_FEATURESETHEADER']._serialized_start=83
  _globals['_FEATURESETHEADER']._serialized_end=174
# @@protoc_insertion_point(module_scope)
