# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ai/h2o/featurestore/api/v1/RecommendationProtoApi.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n7ai/h2o/featurestore/api/v1/RecommendationProtoApi.proto\x12\x1a\x61i.h2o.featurestore.api.v1\"L\n!RecommendationRegexMatchingPolicy\x12\r\n\x05regex\x18\x01 \x01(\t\x12\x18\n\x10percentage_match\x18\x02 \x01(\r\"\xbf\x01\n\"RecommendationSampleMatchingPolicy\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\x12!\n\x19\x66\x65\x61ture_set_major_version\x18\x02 \x01(\r\x12\x13\n\x0b\x63olumn_name\x18\x03 \x01(\t\x12\x17\n\x0fsample_fraction\x18\x04 \x01(\x01\x12\x16\n\x0e\x66uzzy_distance\x18\x05 \x01(\r\x12\x18\n\x10percentage_match\x18\x06 \x01(\r\"q\n$ListRecommendationClassifierResponse\x12I\n\x0b\x63lassifiers\x18\x01 \x03(\x0b\x32\x34.ai.h2o.featurestore.api.v1.RecommendationClassifier\"\xdd\x01\n\x18RecommendationClassifier\x12\x0c\n\x04name\x18\x01 \x01(\t\x12N\n\x05regex\x18\x02 \x01(\x0b\x32=.ai.h2o.featurestore.api.v1.RecommendationRegexMatchingPolicyH\x00\x12P\n\x06sample\x18\x03 \x01(\x0b\x32>.ai.h2o.featurestore.api.v1.RecommendationSampleMatchingPolicyH\x00\x42\x11\n\x0fmatching_policy\"q\n%CreateRecommendationClassifierRequest\x12H\n\nclassifier\x18\x01 \x01(\x0b\x32\x34.ai.h2o.featurestore.api.v1.RecommendationClassifier\"q\n%UpdateRecommendationClassifierRequest\x12H\n\nclassifier\x18\x01 \x01(\x0b\x32\x34.ai.h2o.featurestore.api.v1.RecommendationClassifier\"@\n%DeleteRecommendationClassifierRequest\x12\x17\n\x0f\x63lassifier_name\x18\x01 \x01(\tB\x1d\x42\x1bRecommendationProtoApiProtob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ai.h2o.featurestore.api.v1.RecommendationProtoApi_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\033RecommendationProtoApiProto'
  _globals['_RECOMMENDATIONREGEXMATCHINGPOLICY']._serialized_start=87
  _globals['_RECOMMENDATIONREGEXMATCHINGPOLICY']._serialized_end=163
  _globals['_RECOMMENDATIONSAMPLEMATCHINGPOLICY']._serialized_start=166
  _globals['_RECOMMENDATIONSAMPLEMATCHINGPOLICY']._serialized_end=357
  _globals['_LISTRECOMMENDATIONCLASSIFIERRESPONSE']._serialized_start=359
  _globals['_LISTRECOMMENDATIONCLASSIFIERRESPONSE']._serialized_end=472
  _globals['_RECOMMENDATIONCLASSIFIER']._serialized_start=475
  _globals['_RECOMMENDATIONCLASSIFIER']._serialized_end=696
  _globals['_CREATERECOMMENDATIONCLASSIFIERREQUEST']._serialized_start=698
  _globals['_CREATERECOMMENDATIONCLASSIFIERREQUEST']._serialized_end=811
  _globals['_UPDATERECOMMENDATIONCLASSIFIERREQUEST']._serialized_start=813
  _globals['_UPDATERECOMMENDATIONCLASSIFIERREQUEST']._serialized_end=926
  _globals['_DELETERECOMMENDATIONCLASSIFIERREQUEST']._serialized_start=928
  _globals['_DELETERECOMMENDATIONCLASSIFIERREQUEST']._serialized_end=992
# @@protoc_insertion_point(module_scope)
