# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ai/h2o/featurestore/api/v1/Events.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'ai/h2o/featurestore/api/v1/Events.proto\x12\x1a\x61i.h2o.featurestore.api.v1\"o\n\x11ModificationEvent\x12Q\n\x13\x66\x65\x61ture_set_deleted\x18\x01 \x01(\x0b\x32\x32.ai.h2o.featurestore.api.v1.FeatureSetDeletedEventH\x00\x42\x07\n\x05value\"0\n\x16\x46\x65\x61tureSetDeletedEvent\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\tB\rB\x0b\x45ventsProtob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ai.h2o.featurestore.api.v1.Events_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\013EventsProto'
  _globals['_MODIFICATIONEVENT']._serialized_start=71
  _globals['_MODIFICATIONEVENT']._serialized_end=182
  _globals['_FEATURESETDELETEDEVENT']._serialized_start=184
  _globals['_FEATURESETDELETEDEVENT']._serialized_end=232
# @@protoc_insertion_point(module_scope)
